!           Binary lens system important calculations               !
!-------------------------------------------------------------------!
!
!  1  get_bin_cau_js_qs(caustic_, critical_, q, s, nsteps)      - calculates points on caustics and critical curves in CMS
!  2  get_bin_cau_js_m(caustic_, critical_, z1, z2, m1, m2, nsteps)      - calculates points on caustics and critical curves (no assumptions)
!
! Jan Skowron (2014) GNU General Public License (GPLv3) 
! email: jskowron@astrouw.edu.pl
! address: Warsaw University Observatory
!          Al. Ujazdowskie 4, Warsaw, Poland
!-------------------------------------------------------------------!




!-------------------------------------------------------------------!
! _1_                   GET_BIN_CAU_JS_QS                           !
!-------------------------------------------------------------------!
subroutine get_bin_cau_js_qs(caustic, critical, q, s, nsteps)
  ! in Center of Mass coordinate system
  implicit none
  ! FUNCTION ARGUMENTS
  integer, parameter :: RK = 8  ! kind for real and complex variables
  real(kind=RK), intent(in) :: q, s
  integer, intent(in) :: nsteps
  integer, parameter :: degree = 4
  complex(kind=RK), intent(out), dimension(nsteps,degree) :: caustic, critical

  ! TEMPORARY
  complex(kind=RK) :: z1, z2
  real(kind=RK) :: m1, m2

  m1 = 1d0/(1d0+q)
  m2 = q*m1
  z1 = dcmplx(-m2*s, 0d0)  ! center of mass is in 0,0
  z2 = dcmplx(m1*s, 0d0)
  call get_bin_cau_js_m(caustic, critical, z1, z2, m1, m2, nsteps)
  return
end


!-------------------------------------------------------------------!
! _2_                   GET_BIN_CAU_JS_M                            !
!-------------------------------------------------------------------!
subroutine get_bin_cau_js_m(caustic, critical, z1, z2, m1, m2, nsteps)
  implicit none
  ! FUNCTION ARGUMENTS
  integer, parameter :: degree = 4
  integer, parameter :: RK = 8  ! kind for real and complex variables
  real(kind=RK), intent(in) :: m1, m2 ! lens masses
  complex(kind=RK), intent(in) :: z1, z2 ! lens positions
  integer, intent(in) :: nsteps
  complex(kind=RK), intent(out), dimension(nsteps,degree) :: caustic, critical

  ! TEMPORARY
  complex(kind=RK) :: eiphi, zs, zc, z1_bar, z2_bar, z_bar, z, zc2
  real(kind=RK) :: phi, msum, phi_step
  complex(kind=RK), dimension(degree+1) :: poly
  complex(kind=RK), dimension(degree) :: roots
  real(kind=RK), parameter :: pi = 3.141592653589793d0
  integer :: i, r
  ! m1/(z-z1)**2 + m2/(z-z2)**2 = e^(-i*phi) = u
  ! lets move coordinate system to position where z1 + z2 == 0
  zs = (z1 + z2)*0.5d0
  zc = (z1 - z2)*0.5d0
  zc2 = zc*zc
  msum = m1 + m2

  !eiphi = dcmplx(0d0, 0d0)
  phi_step = 2d0*pi/(nsteps)
  phi = 0.5d0*phi_step
  eiphi = dcmplx(dcos(phi), -dsin(phi))
  poly(5) = -eiphi
  poly(4) = dcmplx(0d0, 0d0)
  poly(3) = msum+2*eiphi*zc2
  poly(2) = 2*(m1-m2)*zc
  poly(1) = (msum-eiphi*zc2)*zc2
  call cmplx_roots_gen(roots, poly, degree, .true., .false.) 
  z1_bar = dconjg(z1)
  z2_bar = dconjg(z2)
  do r=1,degree
    z = roots(r) + zs
    critical(1,r) = z
    z_bar = dconjg(z)
    caustic(1,r) = z - m1/(z_bar-z1_bar) - m2/(z_bar-z2_bar)
  enddo

  do i=2,nsteps
    phi = (i-0.5d0)*phi_step
    eiphi = dcmplx(dcos(phi), -dsin(phi))
    poly(5) = -eiphi
    poly(3) = msum+2*eiphi*zc2
    poly(1) = (msum-eiphi*zc2)*zc2
    call cmplx_roots_gen(roots, poly, degree, .true., .true.) 
    do r = 1,degree
      z = roots(r) + zs
      critical(i,r) = z
      z_bar = dconjg(z)
      caustic(i,r) = z - m1/(z_bar-z1_bar) - m2/(z_bar-z2_bar)
    enddo
  enddo
  
end

!-------------------------------------------------------------------!
!
!   Copyright (C) 2014  Jan Skowron
!
!   This program is free software: you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation, either version 3 of the License, or
!   (at your option) any later version.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!
!   You should have received a copy of the GNU General Public License
!   along with this program.  If not, see <http://www.gnu.org/licenses/>.
!
!-------------------------------------------------------------------!

